@@ -9,7 +9,7 @@ mod read;
 mod stream;
 mod write;
 pub(crate) use index_iter::IndexIter;
-pub(crate) use read::TreeIter;
+pub(crate) use read::{ChunkVisitor, TreeIter};
 
 /// Trees can be parametrized with the key type and the sequence type. Also, to avoid a dependency
 /// on a link type with all its baggage, we parameterize the link type.
@@ -41,6 +41,8 @@ pub trait TreeTypes: Debug + Send + Sync + Clone + 'static {
         + Summarizable<Self::Summary>;
     /// link type to use over block boundaries
     type Link: Display + Debug + Hash + Eq + Clone + Copy + Send + Sync + DagCbor;
+
+    const NONCE: &'static [u8; 24] = &[0u8; 24];
 }
 
 /// Everything that is needed to read trees
@@ -193,7 +195,7 @@ impl Config {
             max_leaf_count: 10,
             max_key_branches: 4,
             max_summary_branches: 4,
-            zstd_level: 10,
+            zstd_level: 0,
             max_uncompressed_leaf_size: 16 * 1024 * 1024,
         }
     }
@@ -205,7 +207,7 @@ impl Config {
             max_leaf_count: 1 << 14,
             max_summary_branches: 32,
             max_key_branches: 32,
-            zstd_level: 10,
+            zstd_level: 0,
             max_uncompressed_leaf_size: 16 * 1024 * 1024,
         }
     }
