@@ -44,7 +44,6 @@ use crate::{
     CipherOffset, Forest, Secrets,
 };
 use anyhow::{anyhow, Result};
-use derive_more::From;
 use libipld::{
     cbor::{DagCbor, DagCborCodec},
     codec::{Decode, Encode},
@@ -203,13 +202,6 @@ impl<T: TreeTypes> From<BranchIndex<T>> for Index<T> {
     }
 }
 
-/// enum for a leaf or branch index
-#[derive(Debug, From)]
-pub enum IndexRef<'a, T: TreeTypes> {
-    Leaf(&'a LeafIndex<T>),
-    Branch(&'a BranchIndex<T>),
-}
-
 impl<T: TreeTypes> Clone for Index<T> {
     fn clone(&self) -> Self {
         match self {
@@ -220,13 +212,6 @@ impl<T: TreeTypes> Clone for Index<T> {
 }
 
 impl<T: TreeTypes> Index<T> {
-    pub fn as_index_ref(&self) -> IndexRef<T> {
-        match self {
-            Index::Leaf(x) => IndexRef::Leaf(x),
-            Index::Branch(x) => IndexRef::Branch(x),
-        }
-    }
-
     pub fn summarize(&self) -> T::Summary {
         match self {
             Index::Leaf(x) => x.keys.summarize(),
@@ -444,19 +429,21 @@ impl<T: TreeTypes, R> Display for NodeInfo<T, R> {
 
 pub(crate) fn serialize_compressed<T: TreeTypes>(
     key: &chacha20::Key,
+    nonce: &chacha20::XNonce,
     state: &mut CipherOffset,
     items: &[Index<T>],
     level: i32,
 ) -> Result<Vec<u8>> {
     let zs = ZstdDagCborSeq::from_iter(items, level)?;
-    zs.into_encrypted(key, state)
+    zs.into_encrypted(key, nonce, state)
 }
 
 pub(crate) fn deserialize_compressed<T: TreeTypes>(
     key: &chacha20::Key,
+    nonce: &chacha20::XNonce,
     ipld: &[u8],
 ) -> Result<(Vec<Index<T>>, Range<u64>)> {
-    let (seq, byte_range) = ZstdDagCborSeq::decrypt(ipld, key)?;
+    let (seq, byte_range) = ZstdDagCborSeq::decrypt(ipld, key, nonce)?;
     let seq = seq.items::<Index<T>>()?;
     Ok((seq, byte_range))
 }
@@ -525,6 +512,12 @@ impl FromIterator<()> for UnitSeq {
 #[derive(Debug, Clone, DagCbor)]
 pub struct VecSeq<T: DagCbor>(Vec<T>);
 
+impl<T: DagCbor> AsRef<[T]> for VecSeq<T> {
+    fn as_ref(&self) -> &[T] {
+        &self.0
+    }
+}
+
 impl<T: DagCbor + Clone> CompactSeq for VecSeq<T> {
     type Item = T;
     fn get(&self, index: usize) -> Option<T> {
