@@ -1,23 +1,23 @@
 use banyan::{
-    index::{BranchIndex, Index, LeafIndex},
+    index::{BranchIndex, Index, LeafIndex, VecSeq},
     query::{AllQuery, EmptyQuery, OffsetRangeQuery},
     store::{BranchCache, MemStore},
     Config, Forest, Secrets, StreamBuilder, Tree,
 };
-use common::{create_test_tree, txn, IterExt, Key, KeySeq, Sha256Digest, TT};
+use common::{txn, IterExt, Key, KeyRange, KeySeq, Sha256Digest, TestFilter, TestTree, TT};
 use futures::prelude::*;
 use libipld::{cbor::DagCborCodec, codec::Codec, Cid};
 use quickcheck::TestResult;
 use quickcheck_macros::quickcheck;
-use std::{convert::TryInto, iter, ops::Range, str::FromStr};
+use std::{convert::TryInto, iter, str::FromStr};
 
 use crate::common::no_offset_overlap;
 
 mod common;
 
 #[quickcheck]
-fn build_stream(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn build_stream(t: TestTree) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let actual = txn
         .iter_filtered(&tree, AllQuery)
         .map(|res| res.map(|(_, k, v)| (k, v)))
@@ -26,26 +26,29 @@ fn build_stream(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
 }
 
 /// checks that stream_filtered returns the same elements as filtering each element manually
-fn compare_filtered(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn compare_filtered(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let actual = txn
-        .iter_filtered(&tree, OffsetRangeQuery::from(range.clone()))
+        .iter_filtered(&tree, filter.query())
         .collect::<anyhow::Result<Vec<_>>>()?;
     let expected = xs
         .iter()
         .cloned()
         .enumerate()
         .map(|(i, (k, v))| (i as u64, k, v))
-        .filter(|(offset, _, _)| range.contains(offset))
+        .filter(|triple| filter.contains(triple))
         .collect::<Vec<_>>();
+    if actual != expected {
+        println!("{:?} {:?}", actual, expected);
+    }
     Ok(actual == expected)
 }
 
 /// checks that stream_filtered_chunked returns the same elements as filtering each element manually
-fn compare_filtered_chunked(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn compare_filtered_chunked(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let actual = txn
-        .iter_filtered_chunked(&tree, OffsetRangeQuery::from(range.clone()), &|_| ())
+        .iter_filtered_chunked(&tree, filter.query(), &|_| ())
         .map(|chunk_result| match chunk_result {
             Ok(chunk) => chunk.data.into_iter().map(Ok).boxed(),
             Err(cause) => iter::once(Err(cause)).boxed(),
@@ -57,18 +60,15 @@ fn compare_filtered_chunked(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::R
         .cloned()
         .enumerate()
         .map(|(i, (k, v))| (i as u64, k, v))
-        .filter(|(offset, _, _)| range.contains(offset))
+        .filter(|triple| filter.contains(triple))
         .collect::<Vec<_>>();
     Ok(actual == expected)
 }
 /// checks that stream_filtered_chunked returns the same elements as stream_filtered_chunked_reverse
-fn compare_filtered_chunked_with_reverse(
-    xs: Vec<(Key, u64)>,
-    range: Range<u64>,
-) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn compare_filtered_chunked_with_reverse(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let mut reverse = txn
-        .iter_filtered_chunked_reverse(&tree, OffsetRangeQuery::from(range.clone()), &|_| ())
+        .iter_filtered_chunked_reverse(&tree, filter.query(), &|_| ())
         .map(|chunk_result| match chunk_result {
             Ok(chunk) => chunk.data.into_iter().rev().map(Ok).boxed(),
             Err(cause) => iter::once(Err(cause)).boxed(),
@@ -78,7 +78,7 @@ fn compare_filtered_chunked_with_reverse(
     reverse.reverse();
 
     let forward = txn
-        .iter_filtered_chunked(&tree, OffsetRangeQuery::from(range.clone()), &|_| ())
+        .iter_filtered_chunked(&tree, filter.query(), &|_| ())
         .map(|chunk_result| match chunk_result {
             Ok(chunk) => chunk.data.into_iter().map(Ok).boxed(),
             Err(cause) => iter::once(Err(cause)).boxed(),
@@ -90,19 +90,16 @@ fn compare_filtered_chunked_with_reverse(
         .cloned()
         .enumerate()
         .map(|(i, (k, v))| (i as u64, k, v))
-        .filter(|(offset, _, _)| range.contains(offset))
+        .filter(|triple| filter.contains(triple))
         .collect::<Vec<_>>();
     Ok(reverse == forward && forward == expected)
 }
 
 /// checks that stream_filtered_chunked returns the same elements as filtering each element manually
-fn compare_filtered_chunked_reverse(
-    xs: Vec<(Key, u64)>,
-    range: Range<u64>,
-) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn compare_filtered_chunked_reverse(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let actual = txn
-        .iter_filtered_chunked_reverse(&tree, OffsetRangeQuery::from(range.clone()), &|_| ())
+        .iter_filtered_chunked_reverse(&tree, filter.query(), &|_| ())
         .map(|chunk_result| match chunk_result {
             Ok(chunk) => chunk.data.into_iter().rev().map(Ok).boxed(),
             Err(cause) => iter::once(Err(cause)).boxed(),
@@ -115,7 +112,7 @@ fn compare_filtered_chunked_reverse(
         .enumerate()
         .rev()
         .map(|(i, (k, v))| (i as u64, k, v))
-        .filter(|(offset, _, _)| range.contains(offset))
+        .filter(|triple| filter.contains(triple))
         .collect::<Vec<_>>();
     if actual != expected {
         println!("{:?} {:?}", actual, expected);
@@ -124,10 +121,10 @@ fn compare_filtered_chunked_reverse(
 }
 
 /// checks that stream_filtered_chunked returns the same elements as filtering each element manually
-fn filtered_chunked_no_holes(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn filtered_chunked_no_holes(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let chunks = txn
-        .iter_filtered_chunked(&tree, OffsetRangeQuery::from(range), &|_| ())
+        .iter_filtered_chunked(&tree, filter.query(), &|_| ())
         .collect::<anyhow::Result<Vec<_>>>()?;
     let max_offset = chunks.iter().fold(0, |offset, chunk| {
         if offset == chunk.range.start {
@@ -139,9 +136,25 @@ fn filtered_chunked_no_holes(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::
     Ok(max_offset == (xs.len() as u64))
 }
 
-fn iter_index(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
+/// checks that stream_filtered_chunked returns the same elements as filtering each element manually
+fn filtered_chunked_reverse_no_holes(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    let (tree, txn, _xs) = t.tree()?;
+    let chunks = txn
+        .iter_filtered_chunked_reverse(&tree, filter.query(), &|_| ())
+        .collect::<anyhow::Result<Vec<_>>>()?;
+    let min_offset = chunks.iter().fold(tree.count(), |offset, chunk| {
+        if offset == chunk.range.end {
+            chunk.range.start
+        } else {
+            offset
+        }
+    });
+    Ok(min_offset == 0)
+}
+
+fn iter_index(t: TestTree) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let len = xs.len() as u64;
-    let (tree, txn) = create_test_tree(xs)?;
     let actual = txn
         .iter_index(&tree, AllQuery)
         .collect::<anyhow::Result<Vec<_>>>()?;
@@ -158,47 +171,49 @@ fn iter_index(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
 }
 
 #[quickcheck]
-fn build_iter_index(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
-    iter_index(xs)
+fn build_iter_index(t: TestTree) -> anyhow::Result<bool> {
+    iter_index(t)
 }
 
 #[quickcheck]
-fn build_stream_filtered(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::Result<bool> {
-    compare_filtered(xs, range)
+fn build_stream_filtered(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    compare_filtered(t, filter)
 }
 
 #[quickcheck]
-fn build_stream_filtered_chunked(xs: Vec<(Key, u64)>, range: Range<u64>) -> anyhow::Result<bool> {
-    compare_filtered_chunked(xs, range)
+fn build_stream_filtered_chunked(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    compare_filtered_chunked(t, filter)
 }
 
 #[quickcheck]
 fn build_stream_filtered_chunked_forward_and_reverse(
-    xs: Vec<(Key, u64)>,
-    range: Range<u64>,
+    t: TestTree,
+    filter: TestFilter,
 ) -> anyhow::Result<bool> {
-    compare_filtered_chunked_with_reverse(xs, range)
+    compare_filtered_chunked_with_reverse(t, filter)
 }
 
 #[quickcheck]
-fn build_stream_filtered_chunked_reverse(
-    xs: Vec<(Key, u64)>,
-    range: Range<u64>,
-) -> anyhow::Result<bool> {
-    compare_filtered_chunked_reverse(xs, range)
+fn build_stream_filtered_chunked_reverse(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    compare_filtered_chunked_reverse(t, filter)
 }
 
 #[quickcheck]
-fn build_stream_filtered_chunked_no_holes(
-    xs: Vec<(Key, u64)>,
-    range: Range<u64>,
-) -> anyhow::Result<bool> {
-    filtered_chunked_no_holes(xs, range)
+fn build_stream_filtered_chunked_no_holes(t: TestTree, filter: TestFilter) -> anyhow::Result<bool> {
+    filtered_chunked_no_holes(t, filter)
 }
 
 #[quickcheck]
-fn build_get(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
-    let (tree, txn) = create_test_tree(xs.clone())?;
+fn build_stream_filtered_chunked_reverse_no_holes(
+    t: TestTree,
+    filter: TestFilter,
+) -> anyhow::Result<bool> {
+    filtered_chunked_reverse_no_holes(t, filter)
+}
+
+#[quickcheck]
+fn build_get(t: TestTree) -> anyhow::Result<bool> {
+    let (tree, txn, xs) = t.tree()?;
     let mut actual = Vec::new();
     for i in 0..xs.len() as u64 {
         actual.push(txn.get(&tree, i)?.unwrap());
@@ -250,42 +265,30 @@ fn build_pack_1() {
     assert!(do_build_pack(xss).unwrap());
 }
 
-fn do_retain(xss: Vec<Vec<(Key, u64)>>) -> anyhow::Result<bool> {
-    let store = MemStore::new(usize::max_value(), Sha256Digest::digest);
-    let forest = txn(store, 1000);
-    let mut builder = StreamBuilder::<TT, u64>::debug();
-    // flattened xss for reference
-    let xs = xss.iter().cloned().flatten().collect::<Vec<_>>();
-    // build complex unbalanced tree
-    for xs in xss.iter() {
-        forest.extend_unpacked(&mut builder, xs.clone()).unwrap();
-    }
+fn do_retain(t: TestTree) -> anyhow::Result<bool> {
+    let (mut builder, txn, xs) = t.builder()?;
     let tree0 = builder.snapshot();
-    forest.retain(&mut builder, &OffsetRangeQuery::from(xs.len() as u64..))?;
+    txn.retain(&mut builder, &OffsetRangeQuery::from(xs.len() as u64..))?;
     let tree1 = builder.snapshot();
-    forest.assert_invariants(&builder)?;
-    forest.pack(&mut builder)?;
+    txn.assert_invariants(&builder)?;
+    txn.pack(&mut builder)?;
     let tree2 = builder.snapshot();
-    forest.retain(&mut builder, &OffsetRangeQuery::from(xs.len() as u64..))?;
-    forest.assert_invariants(&builder)?;
+    txn.retain(&mut builder, &OffsetRangeQuery::from(xs.len() as u64..))?;
+    txn.assert_invariants(&builder)?;
     let tree3 = builder.snapshot();
-    let offsets_ok = no_offset_overlap(&forest, &[tree0, tree1, tree2, tree3])?;
+    let offsets_ok = no_offset_overlap(&txn, &[tree0, tree1, tree2, tree3])?;
     Ok(offsets_ok)
 }
 
 #[quickcheck]
-fn retain(xss: Vec<Vec<(Key, u64)>>) -> anyhow::Result<bool> {
-    do_retain(xss)
+fn retain(t: TestTree) -> anyhow::Result<bool> {
+    do_retain(t)
 }
 
 #[quickcheck]
-fn iter_from_should_return_all_items(xs: Vec<(Key, u64)>) -> anyhow::Result<bool> {
-    let store = MemStore::new(usize::max_value(), Sha256Digest::digest);
-    let forest = txn(store, 1000);
-    let mut builder = StreamBuilder::<TT, u64>::debug();
-    forest.extend(&mut builder, xs.clone().into_iter())?;
-    forest.assert_invariants(&builder)?;
-    let actual = forest
+fn iter_from_should_return_all_items(t: TestTree) -> anyhow::Result<bool> {
+    let (builder, txn, xs) = t.builder()?;
+    let actual = txn
         .iter_from(&builder.snapshot())
         .collect::<anyhow::Result<Vec<_>>>()?;
     let expected = xs
@@ -303,7 +306,21 @@ fn iter_from_should_return_all_items(xs: Vec<(Key, u64)>) -> anyhow::Result<bool
 #[test]
 fn filter_test_simple() -> anyhow::Result<()> {
     let _ = env_logger::builder().is_test(true).try_init();
-    let ok = compare_filtered(vec![(Key(1), 1), (Key(2), 2)], 0..1)?;
+    let ok = compare_filtered(
+        TestTree::packed(vec![(Key(1), 1), (Key(2), 2)]),
+        TestFilter::offset_range(0..1),
+    )?;
+    assert!(ok);
+    Ok(())
+}
+
+#[test]
+fn filter_test_simple_2() -> anyhow::Result<()> {
+    let _ = env_logger::builder().is_test(true).try_init();
+    let ok = compare_filtered(
+        TestTree::unpacked(vec![vec![(Key(1), 1)], vec![(Key(2), 2)]]),
+        TestFilter::offset_range(0..1),
+    )?;
     assert!(ok);
     Ok(())
 }
@@ -352,16 +369,13 @@ async fn stream_test_simple() -> anyhow::Result<()> {
 
 #[quickcheck_async::tokio]
 async fn stream_trees_chunked_should_honour_an_inclusive_upper_bound(
-    xs: Vec<(Key, u64)>,
+    t: TestTree,
 ) -> anyhow::Result<TestResult> {
+    let (builder, forest, xs) = t.builder()?;
     let len = xs.len() as u64;
     if len == 0 {
         return Ok(TestResult::discard());
     }
-    let store = MemStore::new(usize::max_value(), Sha256Digest::digest);
-    let forest = txn(store, 1000);
-    let mut builder = StreamBuilder::<TT, u64>::debug();
-    forest.extend_unpacked(&mut builder, xs.clone().into_iter())?;
     let trees = stream::once(async move { builder.snapshot() }).chain(stream::pending());
 
     let actual = forest
@@ -385,44 +399,53 @@ async fn stream_trees_chunked_should_honour_an_inclusive_upper_bound(
     Ok(TestResult::from_bool(expected == actual))
 }
 
+#[tokio::test]
+async fn stream_trees_chunked_reverse_should_honour_an_inclusive_upper_bound_1(
+) -> anyhow::Result<()> {
+    let t = TestTree::unpacked(vec![vec![(Key(0), 0)], vec![(Key(1), 1)]]);
+    let res = do_stream_trees_chunked_reverse_should_honour_an_inclusive_upper_bound(t).await?;
+    assert!(!res.is_failure());
+    Ok(())
+}
+
 #[quickcheck_async::tokio]
 async fn stream_trees_chunked_reverse_should_honour_an_inclusive_upper_bound(
-    xs: Vec<(Key, u64)>,
+    t: TestTree,
 ) -> anyhow::Result<TestResult> {
+    do_stream_trees_chunked_reverse_should_honour_an_inclusive_upper_bound(t).await
+}
+
+async fn do_stream_trees_chunked_reverse_should_honour_an_inclusive_upper_bound(
+    t: TestTree,
+) -> anyhow::Result<TestResult> {
+    let (builder, forest, xs) = t.builder()?;
     let len = xs.len() as u64;
     if len == 0 {
         return Ok(TestResult::discard());
     }
-    let store = MemStore::new(usize::max_value(), Sha256Digest::digest);
-    let forest = txn(store, 1000);
-    let mut builder = StreamBuilder::<TT, u64>::debug();
-    forest.extend_unpacked(&mut builder, xs.clone().into_iter())?;
     let trees = stream::once(async move { builder.snapshot() }).chain(stream::pending());
     let actual = forest
         .stream_trees_chunked_reverse(AllQuery, trees, 0u64..=(len - 1), &|_| ())
-        .map_ok(move |chunk| stream::iter(chunk.data))
+        .map_ok(move |chunk| {
+            // reverse the chunk
+            stream::iter(chunk.data.into_iter().rev())
+        })
         .take_while(|x| future::ready(x.is_ok()))
         .filter_map(|x| future::ready(x.ok()))
         .flatten()
         .collect::<Vec<_>>()
         .await;
 
-    let values = xs
+    let expected = xs
         .iter()
         .cloned()
         .enumerate()
         .map(|(i, (k, v))| (i as u64, k, v))
-        .collect::<Vec<_>>();
-    let leaf_count = Config::debug().max_leaf_count;
-    // desc per chunk, asc inside chunk
-    let expected: Vec<_> = values
-        .chunks(leaf_count as usize)
         .rev()
-        .flat_map(|x| x.iter().cloned())
-        .collect();
+        .collect::<Vec<_>>();
 
     if expected != actual {
-        println!("{:?} {:?}", expected, actual);
+        println!("expected {:?} actual {:?}", expected, actual);
     }
     Ok(TestResult::from_bool(expected == actual))
 }
@@ -562,7 +585,9 @@ fn branch_index_wire_format() -> anyhow::Result<()> {
         sealed: true,
         key_bytes: 67834,
         value_bytes: 123478912,
-        summaries: KeySeq(vec![Key(1), Key(2)]),
+        summaries: vec![KeyRange(0, 1), KeyRange(1, 2)]
+            .into_iter()
+            .collect::<VecSeq<_>>(),
         link: Some(
             Cid::from_str("bafyreihtx752fmf3zafbys5dtr4jxohb53yi3qtzfzf6wd5274jwtn5agu")?
                 .try_into()?,
@@ -572,38 +597,41 @@ fn branch_index_wire_format() -> anyhow::Result<()> {
     let serialized = DagCborCodec.encode(&index)?;
     let expected = from_cbor_me(
         r#"
-A7                                      # map(7)
-   65                                   # text(5)
-      636F756E74                        # "count"
-   19 8FB0                              # unsigned(36784)
-   69                                   # text(9)
-      6B65795F6279746573                # "key_bytes"
-   1A 000108FA                          # unsigned(67834)
-   65                                   # text(5)
-      6C6576656C                        # "level"
-   03                                   # unsigned(3)
-   64                                   # text(4)
-      6C696E6B                          # "link"
-   D8 2A                                # tag(42)
-      58 25                             # bytes(37)
-         0001711220F3BFFBA2B0BBC80A1C4BA39C789BB8E1EEF08DC2792E4BEB0FBAFF1369B7A035 # "\x00\x01q\x12 \xF3\xBF\xFB\xA2\xB0\xBB\xC8\n\x1CK\xA3\x9Cx\x9B\xB8\xE1\xEE\xF0\x8D\xC2y.K\xEB\x0F\xBA\xFF\x13i\xB7\xA05"
-   66                                   # text(6)
-      7365616C6564                      # "sealed"
-   F5                                   # primitive(21)
-   69                                   # text(9)
-      73756D6D6172696573                # "summaries"
-   81                                   # array(1)
-      82                                # array(2)
-         81                             # array(1)
-            01                          # unsigned(1)
-         81                             # array(1)
-            02                          # unsigned(2)
-   6B                                   # text(11)
-      76616C75655F6279746573            # "value_bytes"
-   1A 075C2380                          # unsigned(123478912)
 
+A7                                      # map(7)
+65                                   # text(5)
+   636F756E74                        # "count"
+19 8FB0                              # unsigned(36784)
+69                                   # text(9)
+   6B65795F6279746573                # "key_bytes"
+1A 000108FA                          # unsigned(67834)
+65                                   # text(5)
+   6C6576656C                        # "level"
+03                                   # unsigned(3)
+64                                   # text(4)
+   6C696E6B                          # "link"
+D8 2A                                # tag(42)
+   58 25                             # bytes(37)
+      0001711220F3BFFBA2B0BBC80A1C4BA39C789BB8E1EEF08DC2792E4BEB0FBAFF1369B7A035 # "\x00\x01q\x12 \xF3\xBF\xFB\xA2\xB0\xBB\xC8\n\x1CK\xA3\x9Cx\x9B\xB8\xE1\xEE\xF0\x8D\xC2y.K\xEB\x0F\xBA\xFF\x13i\xB7\xA05"
+66                                   # text(6)
+   7365616C6564                      # "sealed"
+F5                                   # primitive(21)
+69                                   # text(9)
+   73756D6D6172696573                # "summaries"
+81                                   # array(1)
+   82                                # array(2)
+      82                             # array(2)
+         00                          # unsigned(0)
+         01                          # unsigned(1)
+      82                             # array(2)
+         01                          # unsigned(1)
+         02                          # unsigned(2)
+6B                                   # text(11)
+   76616C75655F6279746573            # "value_bytes"
+1A 075C2380                          # unsigned(123478912
 "#,
     )?;
+    println!("{}", hex::encode(&serialized));
     assert_eq!(serialized, expected);
     Ok(())
 }
@@ -611,7 +639,7 @@ A7                                      # map(7)
 #[test]
 fn retain1() -> anyhow::Result<()> {
     let xs = (0..10).map(|i| (Key(i), i)).collect::<Vec<_>>();
-    let ok = do_retain(vec![xs])?;
+    let ok = do_retain(TestTree::packed(xs))?;
     assert!(ok);
     Ok(())
 }
@@ -763,7 +791,7 @@ fn retain2() -> anyhow::Result<()> {
             (Key(39), 0),
         ],
     ];
-    let ok = do_retain(xs)?;
+    let ok = do_retain(TestTree::unpacked(xs))?;
     assert!(ok);
     Ok(())
 }
